// Copyright 2013 com authors
//
// Licensed under the Apache License, Version 2.0 (the "License"): you may
// not use this file except in compliance with the License. You may obtain
// a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

package com

import (
	"testing"
)

func TestHtml2JS(t *testing.T) {
	htm := "<div id=\"button\" class=\"btn\">Click me</div>\n\r"
	js := string(Html2JS([]byte(htm)))
	jsR := `<div id=\"button\" class=\"btn\">Click me</div>\n`
	if js != jsR {
		t.Errorf("Html2JS:\n Expect => %s\n Got => %s\n", jsR, js)
	}
}

func BenchmarkHtml2JS(b *testing.B) {
	htm := "<div id=\"button\" class=\"btn\">Click me</div>\n\r"
	for i := 0; i < b.N; i++ {
		Html2JS([]byte(htm))
	}
}
